@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2021 Supercomputing Systems AG
+# Copyright (C) 2022 Supercomputing Systems AG
 # This file is part of smartmeter-datacollector.
 #
 # SPDX-License-Identifier: GPL-2.0-only
@@ -50,7 +50,8 @@ def parse_arguments():
     parser.add_argument(
         '-c', '--config', help="File path of the configuration (.ini) file.", default="./datacollector.ini")
     parser.add_argument(
-        '-s', '--saveconfig', help="Create default configuration (.ini) file at path defined with -c", action='store_true')
+        '-s', '--saveconfig', help="Create default configuration (.ini) file at path defined with -c",
+        action='store_true')
     parser.add_argument(
         '-d', '--dev', help="Development mode", action='store_true')
     return parser.parse_args()
